@@ -26,8 +26,7 @@
 #include "gom-repository.h"
 #include "gom-resource.h"
 #include "gom-resource-priv.h"
-
-G_DEFINE_ABSTRACT_TYPE(GomResource, gom_resource, G_TYPE_OBJECT)
+#include "reserved-keywords.h"
 
 struct _GomResourcePrivate
 {
@@ -35,6 +34,8 @@ struct _GomResourcePrivate
    gboolean       is_from_table;
 };
 
+G_DEFINE_ABSTRACT_TYPE_WITH_PRIVATE(GomResource, gom_resource, G_TYPE_OBJECT)
+
 enum
 {
    PROP_0,
@@ -200,6 +201,19 @@ gom_resource_class_set_reference (GomResourceClass     *resource_class,
                                g_strdup(ref_property_name), g_free);
 }
 
+static gboolean
+is_valid_table_name (const gchar *table)
+{
+   guint i;
+
+   for (i = 0; i < G_N_ELEMENTS (reserved_keywords); i++) {
+      if (g_ascii_strcasecmp (reserved_keywords[i], table) == 0)
+         return FALSE;
+   }
+
+   return TRUE;
+}
+
 void
 gom_resource_class_set_table (GomResourceClass *resource_class,
                               const gchar      *table)
@@ -207,6 +221,7 @@ gom_resource_class_set_table (GomResourceClass *resource_class,
    g_return_if_fail(GOM_IS_RESOURCE_CLASS(resource_class));
    g_return_if_fail(table != NULL);
    g_return_if_fail(strlen(table) <= sizeof(resource_class->table));
+   g_return_if_fail(is_valid_table_name(table));
 
    g_snprintf(resource_class->table,
               sizeof(resource_class->table),
@@ -1033,6 +1048,36 @@ gom_resource_set_property (GObject      *object,
    }
 }
 
+static void
+pkey_changed_cb (GObject    *gobject,
+                 GParamSpec *pspec,
+                 gpointer    user_data)
+{
+  GomResource *resource = (GomResource *) gobject;
+
+  /* Did the developer reset the primary key? */
+  if (!has_primary_key(GOM_RESOURCE(resource))) {
+     resource->priv->is_from_table = FALSE;
+  }
+}
+
+static void
+gom_resource_constructed (GObject *object)
+{
+  char *pkey_signal;
+  GomResourceClass *klass;
+
+  /* Monitor the primary key */
+  klass = GOM_RESOURCE_CLASS (G_OBJECT_GET_CLASS(object));
+  g_assert (klass->primary_key[0] != '\0');
+  pkey_signal = g_strdup_printf("notify::%s", klass->primary_key);
+  g_signal_connect (G_OBJECT (object), pkey_signal,
+                    G_CALLBACK (pkey_changed_cb), NULL);
+  g_free(pkey_signal);
+
+  G_OBJECT_CLASS (gom_resource_parent_class)->constructed (object);
+}
+
 /**
  * gom_resource_class_init:
  * @klass: (in): A #GomResourceClass.
@@ -1048,7 +1093,7 @@ gom_resource_class_init (GomResourceClass *klass)
    object_class->finalize = gom_resource_finalize;
    object_class->get_property = gom_resource_get_property;
    object_class->set_property = gom_resource_set_property;
-   g_type_class_add_private(object_class, sizeof(GomResourcePrivate));
+   object_class->constructed = gom_resource_constructed;
 
    gParamSpecs[PROP_REPOSITORY] =
       g_param_spec_object("repository",
@@ -1060,19 +1105,6 @@ gom_resource_class_init (GomResourceClass *klass)
                                    gParamSpecs[PROP_REPOSITORY]);
 }
 
-static void
-pkey_changed_cb (GObject    *gobject,
-                 GParamSpec *pspec,
-                 gpointer    user_data)
-{
-  GomResource *resource = (GomResource *) gobject;
-
-  /* Did the developer reset the primary key? */
-  if (!has_primary_key(GOM_RESOURCE(resource))) {
-     resource->priv->is_from_table = FALSE;
-  }
-}
-
 /**
  * gom_resource_init:
  * @resource: (in): A #GomResource.
@@ -1082,20 +1114,7 @@ pkey_changed_cb (GObject    *gobject,
 static void
 gom_resource_init (GomResource *resource)
 {
-   char *pkey_signal;
-   GomResourceClass *klass;
-
-   resource->priv =
-      G_TYPE_INSTANCE_GET_PRIVATE(resource,
-                                  GOM_TYPE_RESOURCE,
-                                  GomResourcePrivate);
-
-   /* Monitor the primary key */
-   klass = GOM_RESOURCE_CLASS (G_OBJECT_GET_CLASS(resource));
-   pkey_signal = g_strdup_printf("notify::%s", klass->primary_key);
-   g_signal_connect (G_OBJECT (resource), pkey_signal,
-                     G_CALLBACK (pkey_changed_cb), NULL);
-   g_free(pkey_signal);
+   resource->priv = gom_resource_get_instance_private(resource);
 }
 
 gboolean
